/*----------------------------------------------------------
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version
2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA  02111-1307  USA

The original authors of this program are Keith Burnett and
Paul Schlyter. (c) 1998

Conversion into C++ class by Hin Jang (December 5, 2003)
http://www.hinjang.com/gfx/
-----------------------------------------------------------*/

// ----------------------------------------------------------
// Moon.h
// ----------------------------------------------------------

#pragma once
#ifndef __MOON_H__
#define __MOON_H__

#include "Observer.h"

#define _USE_MATH_DEFINES // required by Visual Studio .NET
#include <math.h>         // to access math.h constant M_PI
                          //
                          // those using gcc should use the
                          // -lm option during compilation


class Moon
{

public:

	Moon(void);
	~Moon(void);

	void computeMoonPosition(Observer *);
	double getRightAscension();
	double getDeclination();

private:

	double getDays(int, int, int, double);
	double getAngle(double, double);
	double getRange(double);
	int getInteger(double);

	// the right ascension of the Moon is the angular
	// distance eastward along the celestial equator from
	// the vernal equinox to the intersection of the hour
	// circle that passes through the body; expressed in
	// hours and minutes and seconds (hh.mmss)
	//
	// the declination of the Moon is the angular distance
	// to a point on a celestial object measured north or
	// south from the celestial equator; expressed in
	// degrees (ddd.mmss); used with right ascension to
	// specify positions on the celestial sphere
	//
	// EXAMPLE:
	//
	// latitude: 43.67 degrees
	// longitude: -79.39 degrees
	// date: December 12, 2003
	// time: 00:30:00 UT
	//
	// topocentric right ascension = 7.85358
	//                             = 7 hours, 51 minutes,
	//                               12.8872 seconds
	//
	// topocentric declination = 25.2242
	//                         = 25 degrees, 13 minutes,
	//                           27.1306 seconds
	//

	double rightAscensionGEO;   // observer at the center
	double declinationGEO;      // of the Earth

	double rightAscensionTOPO;  // observer on the surface
	double declinationTOPO;     // of the Earth

};

inline double Moon::getRightAscension()
{

	return rightAscensionTOPO;

}

inline double Moon::getDeclination()
{

	return declinationTOPO;

}

// get the number of days to December 31 0h 2000
// h is UT in decimal hours
// valid for the 20th and 21st centuries
inline double Moon::getDays(int y, int m, int d, double h)
{

	long val = - 7 * (y + (m + 9) / 12) / 4 + 275 * m / 9 + d;
	val += y * 367;
	return (double)(val - 730530.0 + h / 24.0);

}

// return the angle in the range [0, 2M_PI] depending on
// the signs of x and y
inline double Moon::getAngle(double y, double x)
{

	double a;

	a = atan(y / x);
	if (x < 0.0) a += M_PI;
	if (y < 0.0 && x > 0.0) a += (2*M_PI);
	return a;

}

// return an angle in the range [0, 2M_PI]
inline double Moon::getRange(double x)
{

	double b;
	double a;
	double tpi;

	tpi = 2 * M_PI;

	b = x / tpi;
	a = tpi * (b - getInteger(b));
	if (a < 0.0) a += tpi;
	return a;

}

// return the integer part of a double
inline int Moon::getInteger(double x)
{

	int sign;
	int val;

	sign = (x < 0.0) ? -1 : 1;
	val = (int)abs(x);
	return sign * val;

}

#endif // __MOON_H__